# D. Cordier - 2023.
# http://orcid.org/0000-0003-4515-6271
# daniel.cordier@cnrs.fr
#
# [10.5281/zenodo.10391618](https://zenodo.org/uploads/10391618)
# Provided under GPL licence: https://www.gnu.org/licenses/licenses.fr.html#GPL
import numpy as np
import lorenz93 as l93
from scipy.optimize import fsolve

def X_Esm1s3 (phi0):
    """
    D. Cordier - 27 janvier 2027.
    Linear regression on data in Fig. 5 (b) of Zhao et al. (2015).
    Inputs:
      - phi0 -----: packing fraction of the granular medium to which the droplet impacts.
        WARNING --: we recommend only values between 0.55 and 0.61
    Outputs:
      - Zc*/D0 * Es**-1/3 --: cf. Fig. 5 (b) of Zhao et al. (2015), in J**-1/3
    """
    out = -1076.4 * phi0 + 673.38 # Linear regression on data in Fig. 5 (b) of Zhao et al. (2015).
    return out

def EK(rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,rigid,CdEq):
    """
    We compute the kinetic energy of the falling droplet.
    Inputs:
     - rho_liq ------: masse volumique du liquide considéré (kg m^-3).
     - sigma_liq ----: tension superficielle du liquide considéré (N m^-1)
     - rho_air ------: masse volumique de l'air (kg m^-3).
     - visco --------: keyword
                        'GP' ---: the viscosity is given by the law relevant for ideal gases.
                        'LOR' --: the visocity is given by the law use by Lorenz (1993) relevant for N2.
     - Mair ---------: masse molaire des molécules de l'air (kg mol^-1).
     - T ------------: température absolue (K).
     - dim_molecule -: taille des molécules d'air (m).
     - d0 -----------: diamètre de la goutte de liquide, non déformée (m).
     - g ------------: accélération de la pesanteur (m s^-2).
     - rigid --: =0 : gouttes déformées par rapport à la sphère.
                 =1 : gouttes NON-déformées par rapport à la sphère.
     - CdEq ---: type of Cd equation used (cf. Clift p. 111)
    Outputs:
     - ek -----------: kinetic energy of the droplet at terminal velocity (J).
    """
    # Re : Reynolds number of the droplet falling into the air.
    # U0 : terminal velocity of falling droplet into the air.
    # Method described in Lorenz (1993).
    V0 = 12 # First estimation of the researched raindrops terminal viscosity (m/s). 
    Re, U0 = l93.solve_Vlorenz(rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,V0,rigid,CdEq)
    
    # Droplet mass:
    Md = rho_liq * 4/3 * np.pi * (d0/2)**3
    
    # Kinetic energy of the falling droplet:
    ek = 1/2 * Md * U0**2
    
    return ek

def Equa_Zhao2015(X,beta0,EK):
    """
    Equation to be solved for the Zhao et al. (2015)
    The variable/unknown 'X' is the ratio Zc*/D0 (see Zhao et al., 2015) where Zc* is the maximum depth
    of the crater excavated by the droplet and D0 its in flight diameter.
    Inputs:
     - X -----: see lines just above.
     - beta0 -: (Zc*/D0)Es**-1/3 at the chosen packing fraction 'phi0'
     - EK ----: kinetic energy of the droplet at its terminal velocity.
    """
    y = 1/beta0**3 * X**3 - X/(1/2+X) * EK
    return y

def ZcD0_Zhao15(phi0,rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,rigid,CdEq):
    """
    We compute the kinetic energy of the falling droplet.
    Inputs:
     - phi0 ---------: packing fraction of the targeted granular medium.
     - rho_liq ------: masse volumique du liquide considéré (kg m^-3).
     - sigma_liq ----: tension superficielle du liquide considéré (N m^-1)
     - rho_air ------: masse volumique de l'air (kg m^-3).
     - visco --------: keyword
                        'GP' ---: the viscosity is given by the law relevant for ideal gases.
                        'LOR' --: the visocity is given by the law use by Lorenz (1993) relevant for N2.
     - Mair ---------: masse molaire des molécules de l'air (kg mol^-1).
     - T ------------: température absolue (K).
     - dim_molecule -: taille des molécules d'air (m).
     - d0 -----------: diamètre de la goutte de liquide, non déformée (m).
     - g ------------: accélération de la pesanteur (m s^-2).
     - rigid --: =0 : gouttes déformées par rapport à la sphère.
                 =1 : gouttes NON-déformées par rapport à la sphère.
     - CdEq ---: type of Cd equation used (cf. Clift p. 111)
    Outputs:
     - Zc/D0 --: ratio between the maximum depth Zc* of the crater and the initial diameter D0 of the
                 droplet.
    """
    beta0 = X_Esm1s3(phi0)
    Ek    = EK(rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,rigid,CdEq)
    #print (beta0,Ek)
    
    ZcD0_estim = 40 # First estimation of the searched value.
    ZcD0       = fsolve(Equa_Zhao2015,ZcD0_estim,args=(beta0,Ek))
    
    return ZcD0[0]

# Weber number in the liquid:
def We_liq(rho_liq,d,U,sigma):
    """
    Inputs:
      - rho_liq: liquid density (kg m^-3)
      - d: diameter of the droplet (m)
      - U: terminal velocity (m s^-1)
      - sigma: surface tension (N m^-1)
    Outputs:
      - We_liq: Weber number in the droplet.
    """
    return rho_liq*d*U**2/sigma

def DmaxD0_Zhao15(phi0,rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,rigid,CdEq):
    """
    We compute the kinetic energy of the falling droplet.
    Inputs:
     - phi0 ---------: packing fraction of the targeted granular medium.
     - rho_liq ------: masse volumique du liquide considéré (kg m^-3).
     - sigma_liq ----: tension superficielle du liquide considéré (N m^-1)
     - rho_air ------: masse volumique de l'air (kg m^-3).
     - visco --------: keyword
                        'GP' ---: the viscosity is given by the law relevant for ideal gases.
                        'LOR' --: the visocity is given by the law use by Lorenz (1993) relevant for N2.
     - Mair ---------: masse molaire des molécules de l'air (kg mol^-1).
     - T ------------: température absolue (K).
     - dim_molecule -: taille des molécules d'air (m).
     - d0 -----------: diamètre de la goutte de liquide, non déformée (m).
     - g ------------: accélération de la pesanteur (m s^-2).
     - rigid --: =0 : gouttes déformées par rapport à la sphère.
                 =1 : gouttes NON-déformées par rapport à la sphère.
     - CdEq ---: type of Cd equation used (cf. Clift p. 111)
    Outputs:
     - Dmax/D0 --: ratio between the maximum diameter Dmax of the droplet during the impact and
                   its initial diameter D0.
    """
    zcd0 = ZcD0_Zhao15(phi0,rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,rigid,CdEq)
    V0 = 12 # First estimation of the researched raindrops terminal viscosity (m/s). 
    Re_air, U0 = l93.solve_Vlorenz(rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,V0,rigid,CdEq)
    
    We     = We_liq(rho_liq,d0,U0,sigma_liq) # NOT corrected Weber number.
    
    X      = ZcD0_Zhao15(phi0,rho_liq,sigma_liq,rho_air,visco,Mair,T,dim_molecule,d0,g,rigid,CdEq)
    #print (X)
    WeCrux = 1/(1+2*X) * We
    
    dmaxd0 = 0.776 * WeCrux**(1/4) # The value "0.776" has been determined using the Fig. 6 (b) 
                                   # of Zhao 2015.
    
    return We, WeCrux, dmaxd0
